function HVInf = HVC(points)
% Written by Masoud Asadzadeh March 17 2011 at UWaterloo
%% Comments
% This function calculates an indicator for each point in the set of
% "points" based on its contribution on the Hyper volume of set of
% "points". This is calculated as the difference between the Hyper volume
% of points with and without each of the points at the time. 
% e.g. HVInf = HV_Contribution([1 10; 2 9.8; 3 5; 4 4; 8 2; 10 1])
%% calculation
    if size(points, 1) <= size(points, 2) + 1
        HVInf = ones(size(points, 1), 1);
        return;
    end
    max_f = max(points, [], 1);
    min_f = min(points, [], 1);
    ind = find(max_f == min_f);
    max_f(ind) = max_f(ind) + 1;
    MAX_f = repmat(max_f, size(points,1), 1);
    MIN_f = repmat(min_f, size(points,1), 1);
    points = (points - MIN_f) ./ (MAX_f - MIN_f);
    if size(points , 2) <= 4
        HV_Calc = 'exact';
    elseif size(points , 2) > 4
        HV_Calc = 'apprx';
    end
    switch HV_Calc
        case {'exact'}
            HVInf = HV_exact(points);
        case {'apprx'}
            HVInf = HV_apprx(points);
    end
end
function HVInf = HV_exact(points)
    temp = [points, (1 : size(points, 1))'];
    endpointIndx = zeros(1, 2 * size(points,2));
    for i = 1 : size(points,2)
        temp = sortrows(temp,i);
        endpointIndx(2 * (i-1) + 1) = temp(1, end);
        endpointIndx(2 * (i-1) + 2) = temp(end, end);
    end
    endpointIndx = unique(endpointIndx);
    if length(endpointIndx) == size(points, 1)
        HVInf = ones(size(points, 1), 1);
        return
    end
    Index = (1 : size(points, 1))';
    Index(endpointIndx) = [];
    totalHV = Hypervolume_MEX(points);
    HVInf = zeros(size(points, 1), 1);
    for i = 1 : length(Index)
        y = points;
        y(Index(i),:) = [];
        subhv = Hypervolume_MEX(y);
        HVInf(Index(i)) = totalHV - subhv;
    end
    non_endpointIndx = (1 : size(points,1))';
    non_endpointIndx(endpointIndx) = [];
    Y = points(endpointIndx, :);
    X = points(non_endpointIndx, :);
    IDX = dsearchn(X,Y);
    for i = 1 : length(endpointIndx)
        if ~isempty(IDX)
            HVInf(endpointIndx(i)) = HVInf(non_endpointIndx(IDX(i)));
        else
            HVInf(endpointIndx(i)) = 1;
        end
    end
end
function HVInf = HV_apprx(points)
    temp = [points, (1 : size(points, 1))'];
    endpointIndx = zeros(1, 2 * size(points,2));
    for i = 1 : size(points,2)
        temp = sortrows(temp,i);
        endpointIndx(2 * (i-1) + 1) = temp(1, end);
        endpointIndx(2 * (i-1) + 2) = temp(end, end);
    end
    endpointIndx = unique(endpointIndx);
    nrOfSamples = max(10000,2*size(points,1)); %Dictates the accuracy of approximation
    HVInf = hypeIndicatorSampled( points, ones(1, size(points, 2)), nrOfSamples);
    nonZero_Indx = find(HVInf > 0);
    Y = points(endpointIndx, :);
    X = points(nonZero_Indx, :);
    IDX = dsearchn(X,Y);
    for i = 1 : length(endpointIndx)
        if ~isempty(IDX)
            HVInf(endpointIndx(i)) = HVInf(nonZero_Indx(IDX(i)));
        else
            HVInf(endpointIndx(i)) = 1;
        end
    end
end

function F = hypeIndicatorSampled( points, bounds, nrOfSamples)
% HYPEINDICATORSAMPLED calculates the HypE fitness
%   HYPEINDICATORSAMPLED( POINTS, BOUNDS, K, NROFSAMPLES  ) 
%   samples the HypE fitness values for objective vectors POINTS. 
%
%   POINTS:      objective vectors as rows, all to be minimized
%   BOUNDS:      reference point
%   K:           parameter of HypE
%   NROFSAMPLES: nr of samples to be used
%
%   Example: f =  hypeIndicatorSampled( [1 3; 3 1], [4 4], 1, 10000 )
    [nrP, dim] = size(points);
    F = zeros(nrP, 1);
    
%     alpha = zeros(1,nrP);
    % Commented by MAsadzadeh June 30 2011 for efficiency
%     for i = 1 : k
%         j = 1:i-1;
%         alpha(i) = prod( (k-j) ./ (nrP - j ) )./i;
%     end  
%     alpha(1) = 1;
    
    % Commented by MAsadzadeh June 30 2011 for efficiency
%     if( length(bounds) == 1 )
%         bounds = repmat( bounds,1, dim );
%     end
    
    BoxL = min(points);
    
    S = rand(nrOfSamples,dim)*diag( bounds - BoxL) ...
    + ones( nrOfSamples,dim)*diag(BoxL);

    dominated = zeros( nrOfSamples, 1 );
    dominated_ind = zeros( nrOfSamples, 1 );
%     j = 0;
%     while j < size(S,1)
%         [OnceDom_flag dominating_ind] = isdominated_once(S(j, :), points);
%         if OnceDom_flag == 1
%             F(dominating_ind) = F(dominating_ind) + 1;
%         end
%     end
%     F = F / nrOfSamples;
            
    ROWS = (1:nrOfSamples)';
    for j = 1 : nrP
        B = S - repmat(points(j,:),size(S,1),1);
        ind = find( sum( B >= 0, 2) == dim);
        dominated(ind) = dominated(ind) + 1;
        dominated_ind(ind) = j;
        Index = find(dominated == 2);
        S(Index, :) = [];
        ROWS(Index) = [];
        dominated(Index) = [];
        dominated_ind(Index) = [];
    end

    Index = (1 : size(S,1));
    Logical = logical(dominated == 1);
    ind = Index(Logical)';
    Index = dominated_ind(ind);
    Index = sort(Index);
    for j = 1 : length(Index)
        F(Index(j)) = F(Index(j)) + 1;
    end
    F = F'*prod( bounds - BoxL)/nrOfSamples;
    F = F';
end