function [ND_set, dominance_flag] = ND_check(ND_set, Y, X)
% Written by MAsadzadeh Sept 2008
% This function gets a set of non-dominated solutions and pints in ND_set
% and the new point Y and the new solution X and does the following:
% 1- checkes if dominance_flag = -1: X is dominated
%            if dominance_flag =  0: X is a new non-dominated solution but not dominating
%            if dominance_flag =  1: X is a new non-dominated solution and dominating
% 2- removes solutions in ND_set dominated by X if necessary
% 3- adds Y and X to ND_set if necessary

% MAsadzadeh modified 23 April 2012 to make sure that from two solutions
% with the same objective function values the  new solution is taken and
% the ole solution is removed in the ND_check. 
dominance_flag = 0;
num_objs = length(Y);
i = 0;    %solution counter
while i < size(ND_set, 1)
    i = i + 1;
    num_eql = sum(Y == ND_set(i, 1 : num_objs));
    num_imp = sum(Y < ND_set(i, 1 : num_objs));
    num_deg = sum(Y > ND_set(i, 1 : num_objs));
    if (num_imp == 0) && (num_deg > 0) % X is dominated
        dominance_flag = -1; % X is dominated
        return;
    elseif num_eql == num_objs % Objective functions are the same for X and archived solution i
        ND_set(i,:) = [Y X]; % Replace solution i in ND_set with X
        dominance_flag = 0;  % X is non-dominated
        return;
    elseif num_imp > 0 && num_deg == 0 % X dominates ith solution in the ND_set
        ND_set(i,:) = []; % Remove solution i from ND_set
        i = i - 1;
        dominance_flag = 1;
    end
end
if isempty(ND_set) % If all the solutions in the ND_set were dominated by X
    ND_set = [Y X];
else % If X dominated a portion of solutions in ND_set 
    ND_set(size(ND_set, 1) + 1, :) = [Y X]; % Add the new solution to the end of ND_set
end